"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultEnumProvider = exports.setDefaultEnumProvider = exports.getDefaultTypeProvider = exports.setDefaultTypeProvider = exports.MethodWithZod = exports.MethodWithZodModel = void 0;
const model_from_zod_1 = require("../model-from-zod");
const decorate_with_zod_input_1 = require("./decorate-with-zod-input");
const make_decorator_from_factory_1 = require("./make-decorator-from-factory");
let DEFAULT_TYPE_PROVIDER;
let DEFAULT_ENUM_PROVIDER;
/**
 * Returns a method decorator that is built with `zod` validation object.
 *
 * @export
 * @template T The type of the `zod` validation object.
 * @param {T} input The `zod` validation object.
 * @param {(string | BaseOptions<T> | undefined)} nameOrOptions The name or
 * the options.
 *
 * @param {GraphQLMDF<BaseTypeOptions>} graphqlDecoratorFactory The actual
 * decorator factory function.
 *
 * @param {DynamicZodModelClass<T>} model The dynamically built model class from
 * `zod` validation object.
 *
 * @return {MethodDecorator} A method decorator.
 */
function MethodWithZodModel(input, nameOrOptions, graphqlDecoratorFactory, model) {
    return function _ModelWithZod(target, methodName, descriptor) {
        var _a;
        let newDescriptor = descriptor || {};
        const originalFunction = (_a = descriptor === null || descriptor === void 0 ? void 0 : descriptor.value) !== null && _a !== void 0 ? _a : target[methodName];
        let decorationProps;
        if (typeof nameOrOptions === 'string') {
            decorationProps = {
                zod: { parseToInstance: true },
            };
        }
        else {
            decorationProps = nameOrOptions;
        }
        const decoratedFunction = (0, decorate_with_zod_input_1.decorateWithZodInput)(originalFunction, input, model, decorationProps);
        newDescriptor.value = decoratedFunction;
        if (!descriptor) {
            Object.defineProperty(target, methodName, newDescriptor);
        }
        const methodDecorator = (0, make_decorator_from_factory_1.makeDecoratorFromFactory)(nameOrOptions, graphqlDecoratorFactory, model);
        methodDecorator(target, methodName, newDescriptor);
    };
}
exports.MethodWithZodModel = MethodWithZodModel;
/**
 * Returns a method decorator that is built with `zod` validation object.
 *
 * @export
 * @template T The type of the `zod` validation object.
 * @param {T} input The `zod` validation object.
 * @param {(string | BaseOptions<T> | undefined)} nameOrOptions The name or
 * the options.
 *
 * @param {GraphQLMDF<BaseTypeOptions>} graphqlDecoratorFactory The actual
 * decorator factory function.
 *
 * @return {MethodDecorator} A method decorator.
 */
function MethodWithZod(input, nameOrOptions, graphqlDecoratorFactory) {
    let zodOptions;
    if (typeof nameOrOptions === 'object') {
        zodOptions = nameOrOptions.zod;
    }
    return MethodWithZodModel(input, nameOrOptions, graphqlDecoratorFactory, (0, model_from_zod_1.modelFromZod)(input, zodOptions));
}
exports.MethodWithZod = MethodWithZod;
/**
 * Sets the default type provider for custom GraphQL Scalars.
 *
 * The type name will be calculated and it will be similar to `TypeScript`
 * types such as: `Record<Optional<String>, Array<Number | String>>`.
 *
 * The user will provide custom scalar type to use for that kind of
 * zod validation.
 *
 * @export
 * @param {TypeProvider} fn The type provider.
 */
function setDefaultTypeProvider(fn) {
    DEFAULT_TYPE_PROVIDER = fn;
}
exports.setDefaultTypeProvider = setDefaultTypeProvider;
/**
 * Gets the default type provided set previously
 * via {@link setDefaultTypeProvider}.
 *
 * @export
 * @return {TypeProvider | undefined} The default type provider.
 */
function getDefaultTypeProvider() {
    return DEFAULT_TYPE_PROVIDER;
}
exports.getDefaultTypeProvider = getDefaultTypeProvider;
/**
 * Sets the default enum provider for custom GraphQL Scalars.
 *
 * @export
 * @param {EnumProvider} fn The enum provider.
 */
function setDefaultEnumProvider(fn) {
    DEFAULT_ENUM_PROVIDER = fn;
}
exports.setDefaultEnumProvider = setDefaultEnumProvider;
/**
 * Gets the default enum provided set previously
 * via {@link setDefaultEnumProvider}.
 *
 * @export
 * @return {EnumProvider | undefined} The default enum provider.
 */
function getDefaultEnumProvider() {
    return DEFAULT_ENUM_PROVIDER;
}
exports.getDefaultEnumProvider = getDefaultEnumProvider;
//# sourceMappingURL=common.js.map