"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decorateWithZodInput = void 0;
const class_transformer_1 = require("class-transformer");
const zod_1 = require("zod");
const common_1 = require("@nestjs/common");
/**
 * Decorates a method with given zod validation object.
 *
 * @export
 * @template T The type of the zod validation object.
 * @template F The type of the function that will be replaced.
 * @param {Function} originalFunction The original function which will be
 * replaced.
 *
 * @param {T} input The zod validation object.
 * @param {DynamicZodModelClass<T>} model The dynamically built zod class that
 * has the validations installed.
 *
 * @return {F}
 */
function decorateWithZodInput(originalFunction, input, model, options) {
    return function _modelWithZod(...args) {
        const result = originalFunction.apply(this, args);
        let parseToInstance = true;
        if (typeof (options === null || options === void 0 ? void 0 : options.zod) === 'object') {
            if (typeof options.zod.parseToInstance === 'boolean') {
                parseToInstance = options.zod.parseToInstance;
            }
        }
        if (result instanceof Promise) {
            return result
                .then(output => input.parseAsync(output))
                .then(output => parseToInstance ? (0, class_transformer_1.plainToInstance)(model, output) : output)
                .catch((error) => {
                if (error instanceof zod_1.ZodError) {
                    throw new common_1.BadRequestException(error.issues);
                }
                else {
                    throw error;
                }
            });
        }
        else {
            const parseResult = input.safeParse(result);
            if (parseResult.success) {
                return parseToInstance ? (0, class_transformer_1.plainToInstance)(model, parseResult.data) : parseResult.data;
            }
            else {
                throw new common_1.BadRequestException(parseResult.error.issues);
            }
        }
    };
}
exports.decorateWithZodInput = decorateWithZodInput;
//# sourceMappingURL=decorate-with-zod-input.js.map