import { infer as Infer, ZodTypeAny } from 'zod';
import { PipeTransform, Type } from '@nestjs/common';
import { ArgsOptions } from '@nestjs/graphql';
import { EnumProvider } from '../../types/enum-provider';
import { TypeProvider } from '../../types/type-provider';
type PT<F = any, T = any> = PipeTransform<F, T> | Type<PipeTransform<F, T>>;
type CustomDecoratorOptions = {
    /**
     * Gets the scalar type for given type name.
     *
     * @param {string} typeName The type name corresponding to the zod object.
     * @return {GraphQLScalarType} The scalar type for the zod object.
     */
    getScalarTypeFor?: TypeProvider;
    /**
     * Provides a name for nested classes when they are created dynamically from
     * object properties of zod types.
     *
     * @param {string} parentName The parent class name.
     * @param {string} propertyKey The property key/name.
     * @return {(string | undefined)} The name to set for the class. If
     * any value returned other than a `string`, the class name will be generated
     * automatically.
     *
     * @memberof IModelFromZodOptions
    */
    provideNameForNestedClass?(parentName: string, propertyKey: string): string | undefined;
    /**
     * Gets an enum type for given information.
     *
     * Use this function to prevent creating different enums in GraphQL schema
     * if you are going to use same values in different places.
     *
     * @param {string | undefined} name The parent name that contains the enum
     * type.
     * @param {string} key The property name of the enum.
     * @param {(Record<string, string | number>)} enumObject The enum object
     * that is extracted from the zod.
     * @return {(Record<string, string | number> | undefined)} The enum
     * that will be used instead of creating a new one. If `undefined` is
     * returned, then a new enum will be created.
     *
     * @memberof IModelFromZodOptions
     */
    getEnumType?: EnumProvider;
};
type DecoratorOptions = ArgsOptions & CustomDecoratorOptions;
/**
 * A parameter decorator that takes a `zod` validation input and marks it as
 * GraphQL `Args` with `property` name with given `options` and pipes.
 *
 * @export
 * @template T The type of the `zod` validation input.
 * @param {T} input The `zod` validation schema object.
 * @param {string} property The name of the property for the GraphQL request
 * argument.
 *
 * @param {DecoratorOptions} options The options for {@link Args} decorator.
 * @param {...PT[]} pipes The pipes that will be passed to {@link Args}
 * decorator.
 *
 * @return {ParameterDecorator} A {@link ParameterDecorator} for GraphQL
 * argument.
 */
export declare function ZodArgs<T extends ZodTypeAny>(input: T, property: string, options: DecoratorOptions, ...pipes: PT[]): ParameterDecorator;
/**
 * A parameter decorator that takes a `zod` validation input and marks it as
 * GraphQL `Args` with `property` name with given `options` and pipes.
 *
 * @export
 * @template T The type of the `zod` validation input.
 * @param {T} input The `zod` validation schema object.
 * @param {DecoratorOptions} options The options for {@link Args} decorator.
 * @param {...PT[]} pipes The pipes that will be passed to {@link Args}
 * decorator.
 *
 * @return {ParameterDecorator} A {@link ParameterDecorator} for GraphQL
 * argument.
 */
export declare function ZodArgs<T extends ZodTypeAny>(input: T, options: DecoratorOptions, ...pipes: PT[]): ParameterDecorator;
/**
 * A parameter decorator that takes a `zod` validation input and marks it as
 * GraphQL `Args` with `property` name with given `options` and pipes.
 *
 * @export
 * @template T The type of the `zod` validation input.
 * @param {T} input The `zod` validation schema object.
 * @param {string} property The name of the property for the GraphQL request
 * argument.
 *
 * @param {...PT[]} pipes The pipes that will be passed to {@link Args}
 * decorator.
 *
 * @return {ParameterDecorator} A {@link ParameterDecorator} for GraphQL
 * argument.
 */
export declare function ZodArgs<T extends ZodTypeAny>(input: T, property: string, ...pipes: PT[]): ParameterDecorator;
/**
 * A parameter decorator that takes a `zod` validation input and marks it as
 * GraphQL `Args` with `property` name with given `options` and pipes.
 *
 * @export
 * @template T The type of the `zod` validation input.
 * @param {T} input The `zod` validation schema object.
 * @param {...PT[]} pipes The pipes that will be passed to {@link Args}
 * decorator.
 *
 * @return {ParameterDecorator} A {@link ParameterDecorator} for GraphQL
 * argument.
 */
export declare function ZodArgs<T extends ZodTypeAny>(input: T, ...pipes: PT[]): ParameterDecorator;
export declare module ZodArgs {
    /**
     * A type for inferring the type of a given `zod` validation object.
     */
    type Of<T extends ZodTypeAny> = Infer<T>;
    /**
     * Frees the used objects during the startup.
     *
     * The {@link ZodArgs} decorator uses helper local variables to keep the
     * naming system working when the same scheme is used multiple times in
     * separate decorators and same name for different schemes.
     *
     * This function should be called after the GraphQL scheme is created.
     *
     * @export
     */
    function free(): void;
}
export {};
