"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZodArgs = void 0;
const zod_1 = require("zod");
const graphql_1 = require("@nestjs/graphql");
const helpers_1 = require("../../helpers");
const get_description_1 = require("../../helpers/get-description");
const get_field_info_from_zod_1 = require("../../helpers/get-field-info-from-zod");
const is_zod_instance_1 = require("../../helpers/is-zod-instance");
const zod_validator_pipe_1 = require("../../helpers/zod-validator.pipe");
const common_1 = require("../common");
const input_from_zod_1 = require("./input-from-zod");
let GENERATED_TYPES;
let USED_NAMES;
/**
 * Creates a new type from given zod object or returns previously created one.
 *
 * @template T The type of the zod object passed.
 * @param {T} input The zod scheme object.
 * @param {CustomDecoratorOptions} options The custom decorator options.
 * @return {*} The newly or previously created class instance.
 */
function _getOrCreateRegisteredType(input, options) {
    if (!GENERATED_TYPES) {
        GENERATED_TYPES = new WeakMap();
    }
    let RegisteredType = GENERATED_TYPES.get(input);
    if (RegisteredType)
        return RegisteredType;
    const { name, description } = (0, helpers_1.extractNameAndDescription)(input, {});
    const safeName = _getSafeName(name);
    RegisteredType = (0, input_from_zod_1.inputFromZod)(input, {
        name: safeName,
        description,
        zod: {
            name: safeName,
            description,
            getEnumType: options.getEnumType,
            getScalarTypeFor: options.getScalarTypeFor,
            provideNameForNestedClass: options.provideNameForNestedClass,
        }
    });
    GENERATED_TYPES.set(input, RegisteredType);
    return RegisteredType;
}
/**
 * Checks if the name is used before, in that case, adds a suffix of `_{number}`
 * indicating the number of times the name is used.
 *
 * @param {string} name The name to check.
 * @return {string} The name that is not used in any other types before.
 */
function _getSafeName(name) {
    if (!USED_NAMES) {
        USED_NAMES = [];
    }
    let total = 0;
    for (let i = 0, limit = USED_NAMES.length; i < limit; ++i) {
        const current = USED_NAMES[i];
        if (current.startsWith(name)) {
            ++total;
        }
    }
    if (total) {
        const newName = `${name}_${total + 1}`;
        USED_NAMES.push(newName);
        return newName;
    }
    USED_NAMES.push(name);
    return name;
}
function ZodArgs(input, propertyOrOptions, optionsOrPipe, ...pipes) {
    var _a, _b;
    let property;
    let options;
    // Parameter normalization
    if (typeof propertyOrOptions === 'string') {
        property = propertyOrOptions;
        if (typeof optionsOrPipe === 'object') {
            if ('transform' in optionsOrPipe) {
                pipes.unshift(optionsOrPipe);
            }
            else {
                options = optionsOrPipe;
            }
        }
    }
    else if (typeof propertyOrOptions === 'object') {
        if (typeof optionsOrPipe === 'object') {
            if ('transform' in optionsOrPipe) {
                pipes.unshift(optionsOrPipe);
            }
            else {
                options = optionsOrPipe;
            }
        }
        else if ('transform' in propertyOrOptions) {
            pipes.unshift(propertyOrOptions);
        }
        else {
            options = propertyOrOptions;
        }
    }
    options !== null && options !== void 0 ? options : (options = {});
    const { getScalarTypeFor = (0, common_1.getDefaultTypeProvider)() } = options;
    if (!(0, is_zod_instance_1.isZodInstance)(zod_1.ZodObject, input)) {
        pipes.unshift(new zod_validator_pipe_1.ZodValidatorPipe(input));
        const typeInfo = (0, get_field_info_from_zod_1.getFieldInfoFromZod)('', input, options);
        const nullability = (0, helpers_1.getNullability)(typeInfo);
        const description = (0, get_description_1.getDescription)(input);
        const { type } = typeInfo;
        options.type = () => type;
        options.nullable = nullability;
        (_a = options.description) !== null && _a !== void 0 ? _a : (options.description = description);
    }
    else {
        const RegisteredType = _getOrCreateRegisteredType(input, {
            getScalarTypeFor
        });
        pipes.unshift(new zod_validator_pipe_1.ZodValidatorPipe(input, RegisteredType));
        (_b = options.type) !== null && _b !== void 0 ? _b : (options.type = () => RegisteredType);
    }
    if (options.name) {
        return prepareDecorator(property, options, ...pipes);
    }
    else {
        return function _anonymousZodArgsWrapper(target, propKey, index) {
            options !== null && options !== void 0 ? options : (options = {});
            options.name = `arg_${index}`;
            const decorator = prepareDecorator(property, options, ...pipes);
            decorator(target, propKey, index);
        };
    }
}
exports.ZodArgs = ZodArgs;
/**
 * Gets a prepared {@link ParameterDecorator} after {@link Args} is called.
 *
 * @param {string} [property] The property string.
 * @param {DecoratorOptions} [options] The decorator options.
 * @param {...PT[]} pipes The pipes to apply.
 * @return {ParameterDecorator} The built parameter decorator.
 */
function prepareDecorator(property, options, ...pipes) {
    let args;
    if (typeof property === 'string') {
        if (typeof options === 'object') {
            args = (0, graphql_1.Args)(property, options, ...pipes);
        }
        else {
            args = (0, graphql_1.Args)(property, ...pipes);
        }
    }
    else if (typeof options === 'object') {
        args = (0, graphql_1.Args)(options, ...pipes);
    }
    else {
        args = (0, graphql_1.Args)(...pipes);
    }
    return args;
}
(function (ZodArgs) {
    /**
     * Frees the used objects during the startup.
     *
     * The {@link ZodArgs} decorator uses helper local variables to keep the
     * naming system working when the same scheme is used multiple times in
     * separate decorators and same name for different schemes.
     *
     * This function should be called after the GraphQL scheme is created.
     *
     * @export
     */
    function free() {
        USED_NAMES = undefined;
        GENERATED_TYPES = undefined;
    }
    ZodArgs.free = free;
})(ZodArgs = exports.ZodArgs || (exports.ZodArgs = {}));
//# sourceMappingURL=zod-args.js.map