"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildEnumType = void 0;
const zod_1 = require("zod");
const graphql_1 = require("@nestjs/graphql");
const common_1 = require("../decorators/common");
const constants_1 = require("./constants");
const is_zod_instance_1 = require("./is-zod-instance");
const to_title_case_1 = require("./to-title-case");
const with_suffix_1 = require("./with-suffix");
/**
 * Builds an enum type for GraphQL schema.
 *
 * @export
 * @template T The type of the zod object.
 * @param {keyof zod.infer<T>} key The key of the zod object.
 * @param {ZodTypeInfo} typeInfo The parsed zod type info.
 * @param {IModelFromZodOptions<zod.infer<T>>} options The options for building
 * enum type.
 *
 * @return {object} The enum object.
 */
function buildEnumType(key, typeInfo, options) {
    var _a, _b;
    const { type } = typeInfo;
    const isNative = (0, is_zod_instance_1.isZodInstance)(zod_1.ZodNativeEnum, type);
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodEnum, type) || isNative) {
        const Enum = isNative ? type.enum : type.Enum;
        let enumProvider = (_a = options.getEnumType) !== null && _a !== void 0 ? _a : (0, common_1.getDefaultEnumProvider)();
        if (typeof enumProvider === 'function') {
            const replacement = enumProvider(Enum, {
                isNative,
                name: String(key),
                parentName: options.name,
                description: type.description,
            });
            if (typeof replacement === 'object' && Enum !== replacement) {
                return typeInfo.type = replacement;
            }
        }
        const incompatibleKey = getFirstIncompatibleEnumKey(Enum);
        if (incompatibleKey) {
            throw new Error(`The value of the Key("${incompatibleKey}") of ${options.name}.${String(key)} Enum was not valid`);
        }
        const parentName = options.name;
        const enumName = (0, with_suffix_1.withSuffix)('Enum')((0, to_title_case_1.toTitleCase)(key));
        const registerCount = (0, constants_1.getRegisterCount)();
        (0, graphql_1.registerEnumType)(Enum, {
            name: (0, to_title_case_1.toTitleCase)(`${parentName}_${enumName}_${registerCount}`),
            description: (_b = type.description) !== null && _b !== void 0 ? _b : `Enum values for ${options.name}.${String(key)}`,
        });
        return typeInfo.type = Enum;
    }
    else if (Array.isArray(type)) {
        const dynamicEnumClass = buildEnumType(key, {
            type: type[0],
            isNullable: !!typeInfo.isItemNullable,
            isOptional: !!typeInfo.isItemOptional,
        }, options);
        return typeInfo.type = [dynamicEnumClass];
    }
    else {
        throw new Error(`Unexpected enum type for Key("${String(key)}")`);
    }
}
exports.buildEnumType = buildEnumType;
function getFirstIncompatibleEnumKey(input) {
    const digitTest = /^\s*?\d/;
    for (const key in input) {
        const value = input[key];
        if (typeof value !== 'string')
            return key;
        if (digitTest.test(value))
            return key;
    }
}
//# sourceMappingURL=build-enum-type.js.map