import { ZodArray, ZodBoolean, ZodDefault, ZodEnum, ZodNativeEnum, ZodNullable, ZodNumber, ZodObject, ZodOptional, ZodString, ZodTransformer, ZodType, ZodTypeAny } from 'zod';
import { IModelFromZodOptions } from '../model-from-zod';
/**
 * Describes the properties of a zod type that can be used to apply to `Field`
 * decorator of NestJS.
 *
 * @export
 * @interface ZodTypeInfo
 */
export interface ZodTypeInfo {
    /**
     * The corresponing type of the `zod` property.
     *
     * This type will be used by the `Field` property of the NestJS decorators.
     *
     * @type {*}
     * @memberof ZodTypeInfo
     */
    type: any;
    /**
     * Indicates whether or not the prperty is optional.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isOptional: boolean;
    /**
     * Indicates whether or not the property is nullable.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isNullable: boolean;
    /**
     * Indicates whether or not the property is an enum type.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isEnum?: boolean;
    /**
     * Indicates whether or not the property is an object (another type).
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isType?: boolean;
    /**
     * Indicates whether or not the property is an array.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isOfArray?: boolean;
    /**
     * Indicates whether or not the item of the array of the property is
     * optional.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isItemOptional?: boolean;
    /**
     * Indicates whether or not the item of the array of the property is
     * nullable.
     *
     * @type {boolean}
     * @memberof ZodTypeInfo
     */
    isItemNullable?: boolean;
}
/**
 * The options for {@link getFieldInfoFromZod} function.
 *
 * The options extends {@link IModelFromZodOptions<T>} because it may create
 * instances if the given zod type was an object, therefore a class would
 * be created.
 *
 * @template T The zod type.
 */
type Options<T extends ZodTypeAny> = IModelFromZodOptions<T> & {
    /**
     * Provides the decorator to decorate the dynamically generated class.
     *
     * @param {T} zodInput The zod input.
     * @param {string} key The name of the currently processsed property.
     * @return {ClassDecorator} The class decorator to decorate the class.
     * @memberof IOptions
     */
    getDecorator?(zodInput: T, key: string): ClassDecorator;
};
/**
 * Converts a given `zod` object input for a key, into {@link ZodTypeInfo}.
 *
 * @export
 * @template T The type of the `zod` object input.
 * @param {string} key The key of the property of the `zod` object input,
 * that is being converted.
 *
 * @param {ZodTypeAny} prop The `zod` object property.
 * @param {Options<T>} options The options for conversion.
 * @return {ZodTypeInfo} The {@link ZodTypeInfo} of the property.
 */
export declare function getFieldInfoFromZod<T extends ZodTypeAny>(key: string, prop: ZodTypeAny, options: Options<T>): ZodTypeInfo;
export declare module getFieldInfoFromZod {
    /**
     * The types that are parseable by the {@link getFieldInfoFromZod} function.
     */
    const PARSED_TYPES: readonly [typeof ZodArray, typeof ZodBoolean, typeof ZodDefault, typeof ZodEnum, typeof ZodNativeEnum, typeof ZodNullable, typeof ZodNumber, typeof ZodObject, typeof ZodOptional, typeof ZodString, typeof ZodTransformer];
    /**
     * Determines if the given zod type is parseable by the {@link getFieldInfoFromZod}
     * function.
     *
     * @export
     * @param {ZodType} input The zod type input.
     * @return {boolean} `true` if the given input is parseable.
     */
    function canParse(input: ZodType): boolean;
}
export {};
