"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFieldInfoFromZod = void 0;
const definition_1 = require("graphql/type/definition");
const zod_1 = require("zod");
const graphql_1 = require("@nestjs/graphql");
const common_1 = require("../decorators/common");
const model_from_zod_1 = require("../model-from-zod");
const get_zod_object_name_1 = require("./get-zod-object-name");
const is_zod_instance_1 = require("./is-zod-instance");
const to_title_case_1 = require("./to-title-case");
/**
 * Converts a given `zod` object input for a key, into {@link ZodTypeInfo}.
 *
 * @export
 * @template T The type of the `zod` object input.
 * @param {string} key The key of the property of the `zod` object input,
 * that is being converted.
 *
 * @param {ZodTypeAny} prop The `zod` object property.
 * @param {Options<T>} options The options for conversion.
 * @return {ZodTypeInfo} The {@link ZodTypeInfo} of the property.
 */
function getFieldInfoFromZod(key, prop, options) {
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodArray, prop)) {
        const data = getFieldInfoFromZod(key, prop.element, options);
        const { type, isEnum, isNullable: isItemNullable, isOptional: isItemOptional, } = data;
        return {
            type: [type],
            isOptional: prop.isOptional(),
            isNullable: prop.isNullable(),
            isEnum,
            isOfArray: true,
            isItemNullable,
            isItemOptional,
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodBoolean, prop)) {
        return {
            type: Boolean,
            isOptional: prop.isOptional(),
            isNullable: prop.isNullable(),
        };
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodString, prop)) {
        return {
            type: String,
            isOptional: prop.isOptional(),
            isNullable: prop.isNullable(),
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNumber, prop)) {
        const isInt = Boolean(prop._def.checks.find((check) => check.kind === 'int'));
        return {
            type: isInt ? graphql_1.Int : Number,
            isOptional: prop.isOptional(),
            isNullable: prop.isNullable(),
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodOptional, prop)) {
        const { type, isEnum, isOfArray, isItemNullable, isItemOptional, } = getFieldInfoFromZod(key, prop.unwrap(), options);
        return {
            type,
            isEnum,
            isOfArray,
            isItemNullable,
            isItemOptional,
            isOptional: true,
            isNullable: prop.isNullable(),
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodObject, prop)) {
        const isNullable = prop.isNullable() || prop.isOptional();
        const { provideNameForNestedClass = defaultNestedClassNameProvider, } = options;
        let name = provideNameForNestedClass(options.name || '', key);
        if (typeof name !== 'string') {
            name = defaultNestedClassNameProvider(options.name || '', key);
        }
        name = name.trim();
        if (!name) {
            name = defaultNestedClassNameProvider(options.name || '', key);
        }
        const nestedOptions = Object.assign(Object.assign({}, options), { name, description: prop.description, isAbstract: isNullable });
        let model;
        if (typeof options.getDecorator === 'function') {
            model = (0, model_from_zod_1.modelFromZodBase)(prop, nestedOptions, options.getDecorator(prop, nestedOptions.name));
        }
        else {
            model = (0, model_from_zod_1.modelFromZod)(prop, nestedOptions);
        }
        return {
            type: model,
            isType: true,
            isNullable: prop.isNullable(),
            isOptional: prop.isOptional(),
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodEnum, prop) || (0, is_zod_instance_1.isZodInstance)(zod_1.ZodNativeEnum, prop)) {
        return {
            type: prop,
            isNullable: prop.isNullable(),
            isOptional: prop.isOptional(),
            isEnum: true,
        };
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodDefault, prop)) {
        return getFieldInfoFromZod(key, prop._def.innerType, options);
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodTransformer, prop)) {
        return getFieldInfoFromZod(key, prop.innerType(), options);
    }
    else if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNullable, prop)) {
        return getFieldInfoFromZod(key, prop._def.innerType, options);
    }
    else {
        const { getScalarTypeFor = (0, common_1.getDefaultTypeProvider)() } = options;
        const typeName = (0, get_zod_object_name_1.getZodObjectName)(prop);
        if (typeof getScalarTypeFor === 'function') {
            const scalarType = getScalarTypeFor(typeName);
            let isScalarType = scalarType instanceof definition_1.GraphQLScalarType;
            if (!isScalarType && scalarType) {
                let constructor = scalarType['constructor'];
                if (typeof constructor === 'function' && constructor.name === definition_1.GraphQLScalarType.name) {
                    isScalarType = true;
                }
            }
            if (isScalarType) {
                return {
                    isType: true,
                    type: scalarType,
                    isNullable: prop.isNullable(),
                    isOptional: prop.isOptional(),
                };
            }
            else {
                throw new Error(`The Scalar(Value="${scalarType}", Type="${typeof scalarType}") as Key("${key}") of Type("${typeName}") was not an instance of GraphQLScalarType.`);
            }
        }
        throw new Error(`Unsupported type info of Key("${key}") of Type("${typeName}")`);
    }
}
exports.getFieldInfoFromZod = getFieldInfoFromZod;
(function (getFieldInfoFromZod) {
    /**
     * The types that are parseable by the {@link getFieldInfoFromZod} function.
     */
    getFieldInfoFromZod.PARSED_TYPES = [
        zod_1.ZodArray,
        zod_1.ZodBoolean,
        zod_1.ZodDefault,
        zod_1.ZodEnum,
        zod_1.ZodNativeEnum,
        zod_1.ZodNullable,
        zod_1.ZodNumber,
        zod_1.ZodObject,
        zod_1.ZodOptional,
        zod_1.ZodString,
        zod_1.ZodTransformer,
    ];
    /**
     * Determines if the given zod type is parseable by the {@link getFieldInfoFromZod}
     * function.
     *
     * @export
     * @param {ZodType} input The zod type input.
     * @return {boolean} `true` if the given input is parseable.
     */
    function canParse(input) {
        return getFieldInfoFromZod.PARSED_TYPES.some(it => (0, is_zod_instance_1.isZodInstance)(it, input));
    }
    getFieldInfoFromZod.canParse = canParse;
})(getFieldInfoFromZod = exports.getFieldInfoFromZod || (exports.getFieldInfoFromZod = {}));
/**
 * Provides a name for nested classes.
 *
 * @param {string} parentName The parent class name.
 * @param {string} propertyKey The property key.
 * @return {string} A new name for the new class.
 *
 * @__PURE__
 */
function defaultNestedClassNameProvider(parentName, propertyKey) {
    return `${parentName}_${(0, to_title_case_1.toTitleCase)(propertyKey)}`;
}
//# sourceMappingURL=get-field-info-from-zod.js.map