"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getZodObjectName = void 0;
const zod_1 = require("zod");
const is_zod_instance_1 = require("./is-zod-instance");
const to_title_case_1 = require("./to-title-case");
/**
 * Builds the corresponding zod type name.
 *
 * @export
 * @param {ZodTypeAny} instance The zod type instance.
 * @return {string} A built type name for the input.
 *
 * @__PURE__
 */
function getZodObjectName(instance) {
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodArray, instance)) {
        const innerName = getZodObjectName(instance.element);
        return `Array<${innerName}>`;
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodOptional, instance)) {
        const innerName = getZodObjectName(instance.unwrap());
        return `Optional<${innerName}>`;
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodTransformer, instance)) {
        return getZodObjectName(instance.innerType());
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodDefault, instance)) {
        return getZodObjectName(instance._def.innerType);
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodEnum, instance)) {
        const { description = '', Enum } = instance;
        const nameSeparatorIndex = description.indexOf(':');
        if (nameSeparatorIndex > 0) {
            const name = description.slice(0, nameSeparatorIndex);
            return `Enum<${name}>`;
        }
        else {
            const values = Object.values(Enum);
            const name = values.join(',');
            return `Enum<${name}>`;
        }
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodObject, instance)) {
        const { description = '' } = instance;
        const nameSeparatorIndex = description.indexOf(':');
        if (nameSeparatorIndex > 0) {
            const name = description.slice(0, nameSeparatorIndex);
            return name;
        }
        else {
            return `Object`;
        }
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodRecord, instance)) {
        const { keySchema, valueSchema } = instance;
        const keyName = getZodObjectName(keySchema);
        const valueName = getZodObjectName(valueSchema);
        return `Record<${keyName}, ${valueName}>`;
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodEffects, instance)) {
        return getZodObjectName(instance.innerType());
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodLiteral, instance)) {
        const { value } = instance;
        if (typeof value === 'object') {
            if (value === null)
                return `Literal<Null>`;
            let constructor;
            if ('prototype' in value) {
                const prototype = value['prototype'];
                if (typeof prototype === 'object' && prototype && ('constructor' in prototype)) {
                    constructor = prototype['constructor'];
                }
            }
            else if ('constructor' in value) {
                constructor = value['constructor'];
            }
            if (typeof constructor === 'function') {
                return `Literal<${constructor.name}>`;
            }
        }
        return `Literal<${(0, to_title_case_1.toTitleCase)(typeof instance.value)}>`;
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodUnion, instance)) {
        return instance.options.map(getZodObjectName).join(' | ');
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNullable, instance)) {
        const innerName = getZodObjectName(instance._def.innerType);
        return `Nullable<${innerName}>`;
    }
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodBoolean, instance))
        return 'Boolean';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodString, instance))
        return 'String';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNumber, instance))
        return 'Number';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodBigInt, instance))
        return 'BigInt';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodDate, instance))
        return 'Date';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodAny, instance))
        return 'Any';
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNull, instance))
        return 'Null';
    return 'Unknown';
}
exports.getZodObjectName = getZodObjectName;
//# sourceMappingURL=get-zod-object-name.js.map