import type { IModelFromZodOptions } from '../model-from-zod';
import { ZodType } from 'zod';
import { NullableList } from '@nestjs/graphql';
import { ZodTypeInfo } from './get-field-info-from-zod';
/**
 * An interface describing a parsed field.
 */
export interface ParsedField {
    /**
     * The key of the parsed property.
     *
     * @type {string}
     */
    key: string;
    /**
     * The type of the field of the parsed property.
     *
     * Can be used for GraphQL @{@link Field} decorator.
     *
     * @type {*}
     */
    fieldType: any;
    /**
     * The {@link PropertyDescriptor} of the parsed property.
     *
     * @type {PropertyDescriptor}
     */
    descriptor: PropertyDescriptor;
    /**
     * A {@link PropertyDecorator} for decorating fields.
     *
     * @type {PropertyDecorator}
     */
    decorateFieldProperty: PropertyDecorator;
}
type ParseOptions<T extends ZodType> = IModelFromZodOptions<T> & {
    /**
   * Provides the decorator to decorate the dynamically generated class.
   *
   * @param {T} zodInput The zod input.
   * @param {string} key The name of the currently processsed property.
   * @return {ClassDecorator} The class decorator to decorate the class.
   * @memberof IOptions
   */
    getDecorator?(zodInput: T, key: string): ClassDecorator;
};
/**
 * Parses a zod input object with given options.
 *
 * @export
 * @template T The type of the zod object.
 * @param {T} zodInput The zod object input.
 * @param {ParseOptions<T>} [options={}] The options for the parsing.
 * @return {ParsedField[]} An array of {@link ParsedField}.
 */
export declare function parseShape<T extends ZodType>(zodInput: T, options?: ParseOptions<T>): ParsedField[];
/**
 * Gets the nullability of a field from type info.
 *
 * @export
 * @param {ZodTypeInfo} typeInfo The type info.
 * @return {(boolean | NullableList)} The nullability state.
 */
export declare function getNullability(typeInfo: ZodTypeInfo): boolean | NullableList;
export {};
