"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNullability = exports.parseShape = void 0;
const zod_1 = require("zod");
const graphql_1 = require("@nestjs/graphql");
const common_1 = require("../decorators/common");
const build_enum_type_1 = require("./build-enum-type");
const create_zod_property_descriptor_1 = require("./create-zod-property-descriptor");
const generate_defaults_1 = require("./generate-defaults");
const get_description_1 = require("./get-description");
const get_field_info_from_zod_1 = require("./get-field-info-from-zod");
const get_zod_object_name_1 = require("./get-zod-object-name");
const is_zod_instance_1 = require("./is-zod-instance");
/**
 * Parses a zod input object with given options.
 *
 * @export
 * @template T The type of the zod object.
 * @param {T} zodInput The zod object input.
 * @param {ParseOptions<T>} [options={}] The options for the parsing.
 * @return {ParsedField[]} An array of {@link ParsedField}.
 */
function parseShape(zodInput, options = {}) {
    // Parsing an object shape
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodObject, zodInput)) {
        return Object
            .entries(zodInput.shape)
            .map(([key, value]) => parseSingleShape(key, value, options));
    }
    // Parsing a primitive shape
    const parsedShape = parseSingleShape('', zodInput, options);
    return [parsedShape];
}
exports.parseShape = parseShape;
/**
 * Gets the nullability of a field from type info.
 *
 * @export
 * @param {ZodTypeInfo} typeInfo The type info.
 * @return {(boolean | NullableList)} The nullability state.
 */
function getNullability(typeInfo) {
    const { isNullable, isOptional, isOfArray, isItemOptional, isItemNullable, } = typeInfo;
    let nullable = isNullable || isOptional;
    if (isOfArray) {
        if (isItemNullable || isItemOptional) {
            if (nullable) {
                nullable = 'itemsAndList';
            }
            else {
                nullable = 'items';
            }
        }
    }
    return nullable;
}
exports.getNullability = getNullability;
/**
 * Parses a field from given parameters.
 *
 * @template T The zod type that will be parsed.
 * @param {string} key The proprety key of the zod type.
 * @param {T} input The zod type input.
 * @param {ParseOptions<T>} options The options for parsing.
 * @return {ParsedField} The parsed field output.
 */
function parseSingleShape(key, input, options) {
    const elementType = (0, get_field_info_from_zod_1.getFieldInfoFromZod)(key, input, options);
    const { isEnum } = elementType;
    if (isEnum) {
        (0, build_enum_type_1.buildEnumType)(key, elementType, options);
    }
    const { type: fieldType } = elementType;
    let defaultValue = elementType.isType ? undefined : (0, generate_defaults_1.generateDefaults)(input);
    const nullable = getNullability(elementType);
    if (nullable === 'items') {
        defaultValue = undefined;
    }
    const description = (0, get_description_1.getDescription)(input);
    const descriptor = buildPropertyDescriptor(key, input, options);
    return {
        key,
        fieldType,
        descriptor,
        decorateFieldProperty: (0, graphql_1.Field)(() => fieldType, {
            name: key,
            nullable,
            defaultValue,
            description,
        })
    };
}
/**
 * Creates a property descriptor for given parameters.
 *
 * @param {string} key The key of the input in its object.
 * @param {ZodType} input The zod type input.
 * @param {ParseOptions<ZodType>} options The parse options.
 * @return {PropertyDescriptor} The property descriptor created for it,
 * if the operation was successful.
 *
 * @throws {Error} - The input was not processable and there was no
 * GraphQLScalar type provided for it.
 */
function buildPropertyDescriptor(key, input, options) {
    if (get_field_info_from_zod_1.getFieldInfoFromZod.canParse(input)) {
        return (0, create_zod_property_descriptor_1.createZodPropertyDescriptor)(key, input, options);
    }
    const { getScalarTypeFor = (0, common_1.getDefaultTypeProvider)() } = options;
    const name = (0, get_zod_object_name_1.getZodObjectName)(input);
    if (typeof getScalarTypeFor == 'function') {
        const scalarType = getScalarTypeFor(name);
        if (typeof scalarType === 'object') {
            return (0, create_zod_property_descriptor_1.createZodPropertyDescriptor)(key, input, options);
        }
    }
    let error = `"${key || name}" could not be processed, a corresponding GraphQL scalar type should be provided.`;
    throw new Error(error);
}
//# sourceMappingURL=parse-shape.js.map