import { ZodArray, ZodCatch, ZodDefault, ZodEffects, ZodLazy, ZodNullable, ZodOptional, ZodPromise, ZodSet, ZodTransformer, ZodType } from 'zod';
import { Prev } from '../types/prev';
/**
 * Unwraps any given zod type by one level.
 *
 * The supported zod wrappers are:
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @template T The zod type.
 */
export type UnwrapNestedZod<T extends ZodType> = T extends ZodArray<infer I> ? I : (T extends ZodOptional<infer I> ? I : (T extends ZodTransformer<infer I> ? I : (T extends ZodDefault<infer I> ? I : (T extends ZodEffects<infer I> ? I : (T extends ZodNullable<infer I> ? I : (T extends ZodCatch<infer I> ? I : (T extends ZodPromise<infer I> ? I : (T extends ZodSet<infer I> ? I : (T extends ZodLazy<infer I> ? I : T)))))))));
/**
 * Unwraps any given zod type recursively.
 *
 * The supported zod wrappers are:
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @template T The zod type.
 * @template Depth The maximum depth to unwrap, default `5`.
 */
export type UnwrapNestedZodRecursive<T extends ZodType, Depth extends number = 5> = [Prev[Depth]] extends [never] ? never : [T] extends [UnwrapNestedZod<T>] ? T : (UnwrapNestedZodRecursive<UnwrapNestedZod<T>, Prev[Depth]>);
/**
 * Unwraps the zod object one level.
 *
 * The supported zod wrappers are:
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @export
 * @template T The type of the input.
 * @param {T} input The zod input.
 * @return {UnwrapNestedZod<T>} The unwrapped zod instance.
 *
 * @__PURE__
 */
export declare function unwrapNestedZod<T extends ZodType>(input: T): UnwrapNestedZod<T>;
/**
 * Unwraps the zob object recursively.
 *
 * @export
 * @template T The type of the input.
 * @template Depth The maximum depth for the recursion, `5` by default.
 * @param {T} input The zod input.
 * @return {UnwrapNestedZodRecursive<T, Depth>} The unwrapped zod instance.
 *
 * @__PURE__
 */
export declare function unwrapNestedZodRecursively<T extends ZodType, Depth extends number = 5>(input: T): UnwrapNestedZodRecursive<T, Depth>;
/**
 * Iterates the zod layers by unwrapping the values of the following types:
 *
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @export
 * @template T The input zod type.
 * @param {T} input The zod input.
 */
export declare function iterateZodLayers<T extends ZodType>(input: T): Generator<ZodType<any, import("zod").ZodTypeDef, any>, void, unknown>;
