"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterateZodLayers = exports.unwrapNestedZodRecursively = exports.unwrapNestedZod = void 0;
const zod_1 = require("zod");
const is_zod_instance_1 = require("./is-zod-instance");
/**
 * Unwraps the zod object one level.
 *
 * The supported zod wrappers are:
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @export
 * @template T The type of the input.
 * @param {T} input The zod input.
 * @return {UnwrapNestedZod<T>} The unwrapped zod instance.
 *
 * @__PURE__
 */
function unwrapNestedZod(input) {
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodArray, input))
        return input.element;
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodCatch, input))
        return input._def.innerType;
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodDefault, input))
        return input._def.innerType;
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodEffects, input))
        return input.innerType();
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodLazy, input))
        return input.schema;
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodNullable, input))
        return input.unwrap();
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodOptional, input))
        return input.unwrap();
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodPromise, input))
        return input.unwrap();
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodSet, input))
        return input._def.valueType;
    if ((0, is_zod_instance_1.isZodInstance)(zod_1.ZodTransformer, input))
        return input.innerType();
    return input;
}
exports.unwrapNestedZod = unwrapNestedZod;
/**
 * Unwraps the zob object recursively.
 *
 * @export
 * @template T The type of the input.
 * @template Depth The maximum depth for the recursion, `5` by default.
 * @param {T} input The zod input.
 * @return {UnwrapNestedZodRecursive<T, Depth>} The unwrapped zod instance.
 *
 * @__PURE__
 */
function unwrapNestedZodRecursively(input) {
    let current = input;
    for (const layer of iterateZodLayers(input)) {
        current = layer;
    }
    return current;
}
exports.unwrapNestedZodRecursively = unwrapNestedZodRecursively;
/**
 * Iterates the zod layers by unwrapping the values of the following types:
 *
 * - {@link ZodArray}
 * - {@link ZodCatch}
 * - {@link ZodDefault}
 * - {@link ZodEffects}
 * - {@link ZodLazy}
 * - {@link ZodNullable}
 * - {@link ZodOptional}
 * - {@link ZodPromise}
 * - {@link ZodSet}
 * - {@link ZodTransformer}
 *
 * @export
 * @template T The input zod type.
 * @param {T} input The zod input.
 */
function* iterateZodLayers(input) {
    let current = input;
    let unwrapped = unwrapNestedZod(input);
    while (unwrapped !== current) {
        yield current;
        current = unwrapped;
        unwrapped = unwrapNestedZod(current);
    }
    yield current;
}
exports.iterateZodLayers = iterateZodLayers;
//# sourceMappingURL=unwrap.js.map