"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.modelFromZod = exports.modelFromZodBase = void 0;
const graphql_1 = require("@nestjs/graphql");
const helpers_1 = require("./helpers");
const constants_1 = require("./helpers/constants");
let _generatedClasses;
/**
 * Creates a dynamic class which will be compatible with GraphQL, from a
 * `zod` model.
 *
 * @export
 * @template T The type of the zod input.
 * @param {T} zodInput The zod object input.
 * @param {IModelFromZodOptions<T>} [options={}] The options for model creation.
 * @return {Type} A class that represents the `zod` object and also
 * compatible with `GraphQL`.
 */
function modelFromZodBase(zodInput, options = {}, decorator) {
    const previousRecord = (_generatedClasses !== null && _generatedClasses !== void 0 ? _generatedClasses : (_generatedClasses = new WeakMap()))
        .get(zodInput);
    if (previousRecord)
        return previousRecord;
    const { name, description } = (0, helpers_1.extractNameAndDescription)(zodInput, options);
    let { keepZodObject = false } = options;
    class DynamicZodModel {
    }
    const prototype = DynamicZodModel.prototype;
    decorator(DynamicZodModel);
    if (keepZodObject) {
        Object.defineProperty(prototype, constants_1.ZodObjectKey, {
            value: Object.assign({}, zodInput),
            configurable: false,
            writable: false,
        });
    }
    const parsed = (0, helpers_1.parseShape)(zodInput, Object.assign(Object.assign({}, options), { name,
        description, getDecorator: options.getDecorator }));
    for (const { descriptor, key, decorateFieldProperty } of parsed) {
        Object.defineProperty(prototype, key, descriptor);
        decorateFieldProperty(prototype, key);
    }
    _generatedClasses.set(zodInput, DynamicZodModel);
    return DynamicZodModel;
}
exports.modelFromZodBase = modelFromZodBase;
/**
 * Creates a dynamic class which will be compatible with GraphQL, from a
 * `zod` model.
 *
 * @export
 * @template T The type of the zod input.
 * @param {T} zodInput The zod object input.
 * @param {IModelFromZodOptions<T>} [options={}] The options for model creation.
 * @return {Type} A class that represents the `zod` object and also
 * compatible with `GraphQL`.
 */
function modelFromZod(zodInput, options = {}) {
    const { name, description } = (0, helpers_1.extractNameAndDescription)(zodInput, options);
    const decorator = (0, graphql_1.ObjectType)(name, Object.assign({ description, isAbstract: zodInput.isNullable() || zodInput.isOptional() }, options));
    return modelFromZodBase(zodInput, options, decorator);
}
exports.modelFromZod = modelFromZod;
//# sourceMappingURL=model-from-zod.js.map